# -*- coding: utf8 -*-
# This code is part of the opsi.org project
# https://www.opsi.org
#
# Copyright (c) 2010-2021 uib GmbH <info@uib.de>
# This code is owned by the uib GmbH, Mainz, Germany (uib.de). All rights reserved.
# License: AGPL-3.0

from pathlib import Path
from typing import Any

from opsisystem.storage import get_disks

class Context:
    def __init__(self) -> None:
        self._context = {}

    @property
    def script_path(self) -> Path:
        return Path("/mnt/opsi") / SCRIPT_PATH

    @property
    def script_path_rw(self) -> Path:
        return Path("/mnt/opsi_rw") / SCRIPT_PATH

    @property
    def depot_script_path(self) -> str:
        return Path(f"/{SCRIPT_PATH}")

    @property
    def patcha_in(self) -> Path:
        return Path(PATCHA_IN)

    def __getattr__(self, name: str) -> Any:
        if not name.startswith("_") and name in self._context:
            return self._context[name]
        raise AttributeError(f"No such attribute in Context: {name!r}")

    def __setattr__(self, name: str, value: Any) -> None:
        if not name.startswith("_"):
            self._context[name] = value
        else:
            super().__setattr__(name, value)


context = Context()


def update_product_on_clients() -> None:
    delete_pocs = []
    for poc in backend.productOnClient_getObjects(clientId = clientId):
        if poc.productType == 'NetbootProduct' and poc.productId != productId:
            delete_pocs.append(poc)
        elif poc.productType == 'LocalbootProduct':
            if poc.installationStatus == 'installed' and poc.productId not in ("opsi-client-agent", "opsi-script", "opsi-winst"):
                backend.setProductActionRequestWithDependencies(poc.productId, clientId, "setup")
            elif poc.installationStatus == 'not_installed' and poc.actionRequest == 'none':
                delete_pocs.append(poc)
    if delete_pocs:
        backend.productOnClient_deleteObjects(delete_pocs)


def set_action_progress(action_progress: str) -> None:
    backend.productOnClient_updateObject({
        "type": "ProductOnClient",
        "productType": "NetbootProduct",
        "productId": productId,
        "clientId": clientId,
        "actionProgress": action_progress
    })

def check_requirements() -> None:
    if not bootimageVersion or (int(bootimageVersion) < 20240228):
        raise RuntimeError("This package requires opsi-linux-bootimage version >= 20240228")


def mount_depot_rw() -> None:
    try:
        depot_rw = getRepository(
        url=depotUrl+'_rw',
        username=username,
        password=pcpatchPassword,
        mountPoint='/mnt/opsi_rw',
        mountOptions={'iocharset': 'utf8', 'vers': '3', 'noserverino': '', 'nobrl': ''}
    )
    except Exception as repositoryError:
        # For backward compatibility
        logger.warning(f"SMB3 mount failed, trying fallback, {repositoryError}")
        depot_rw = getRepository(
            url=depotUrl+'_rw',
            username=username,
            password=pcpatchPassword,
            mountPoint='/mnt/opsi_rw',
            mountOptions={'iocharset': 'utf8', 'vers': '1.0', 'noserverino': '', 'nobrl': ''}
        )
    depot_rw.addHook(uihook)


def get_windows_partition_uefi() -> None:
    disks = sorted(get_disks(), key=lambda disk: disk.dev)
    disk = disks[0]
    context.win_dev = disk.partition_table.get_partition(3)


def get_windows_partition_bios() -> None:
    disks = sorted(get_disks(), key=lambda disk: disk.dev)
    disk = disks[0]
    context.win_dev = disk.partition_table.get_partition(1)


def capture_wim(device: str) -> None:
    replace_existing_image = productPropertyValues.get("replace_existing_image", "False")[0]
    image = productPropertyValues.get("image", [""])[0]
    image_file, image_name = image.split(":", 1) if ":" in image else ("install.wim", image)
    image_path = context.script_path_rw / "images" / image_file
    if not os.path.isfile(f"{image_path}"):
        logger.notice(f"Capture wim {image_path} {image_name} from {device}")
        scriptMessageSubject.setMessage(
            f"Capture {str(image_path)} with image: '{str(image_name.strip())}' from {device}"
        )
        set_action_progress("running wimcapture")
        execute(f"wimcapture {device} {str(image_path)} '{str(image_name.strip())}'")
    else:
        if replace_existing_image:
            logger.notice(f"deleting image {image_name} if it exists")
            try:
                scriptMessageSubject.setMessage(
                    f"Checking if image with same name exists and deleting it"
                )
                execute(f"wimdelete {image_path} {image_name}")
            except Exception as delete_error:
                logger.notice(f"it seems {image_name} didn't exist in {image_file}, {delete_error}")
        logger.notice(f"replace_existing_image is set to {replace_existing_image}")
        logger.notice(f"Append wim {image_file} {image_name} from {device}")
        scriptMessageSubject.setMessage(
            f"Append {str(image_file)} with image: '{str(image_name.strip())}' from {device}"
        )
        set_action_progress("running wimappend")
        execute(f"wimappend {device} {str(image_path)} '{str(image_name.strip())}'")


#mount_depot_rw()
scriptMessageSubject.setMessage(f"mounting opsi_depot_rw")
try:
    depot_rw = getRepository(
    url=depotUrl+'_rw',
    username=username,
    password=pcpatchPassword,
    mountPoint='/mnt/opsi_rw',
    mountOptions={'iocharset': 'utf8', 'vers': '3', 'noserverino': '', 'nobrl': ''}
)
except Exception as repositoryError:
    # For backward compatibility
    logger.warning(f"SMB3 mount failed, trying fallback, {repositoryError}")
    depot_rw = getRepository(
        url=depotUrl+'_rw',
        username=username,
        password=pcpatchPassword,
        mountPoint='/mnt/opsi_rw',
        mountOptions={'iocharset': 'utf8', 'vers': '1.0', 'noserverino': '', 'nobrl': ''}
    )
depot_rw.addHook(uihook)

check_requirements()

scriptMessageSubject.setMessage(f"reading disks")
if inUefi:
    get_windows_partition_uefi()
else:
    get_windows_partition_bios()
time.sleep(10)
capture_wim(context.win_dev.dev)

update_product_on_clients()
set_action_progress("successful")
reboot()
